@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/Xext/security.c,v 1.22 2008/03/18 19:50:45 tsi Exp $ */
+/* $XFree86: xc/programs/Xserver/Xext/security.c,v 1.23 2008/10/15 20:59:11 tsi Exp $ */
 /*
 
 Copyright 1996, 1998  The Open Group
@@ -683,9 +683,12 @@ SProcSecurityGenerateAuthorization(ClientPtr client)
     swaps(&stuff->nbytesAuthProto, n);
     swaps(&stuff->nbytesAuthData, n);
     swapl(&stuff->valueMask, n);
-    values = (CARD32 *)(&stuff[1]) +
-	((stuff->nbytesAuthProto + (unsigned)3) >> 2) +
+    nvalues = ((stuff->nbytesAuthProto + (unsigned)3) >> 2) +
 	((stuff->nbytesAuthData + (unsigned)3) >> 2);
+    if (nvalues >
+	(stuff->length - (sz_xSecurityGenerateAuthorizationReq >> 2)))
+	return BadLength;
+    values = (CARD32 *)(&stuff[1]) + nvalues;
     nvalues = (((CARD32 *)stuff) + stuff->length) - values;
     SwapLongs(values, nvalues);
     return ProcSecurityGenerateAuthorization(client);
