@@ -20,7 +20,7 @@
  * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/sunbw2/bw2_driver.c,v 1.8 2006/03/16 16:50:15 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/sunbw2/bw2_driver.c,v 1.11 2007/03/15 02:11:36 tsi Exp $ */
 
 #include "xf86.h"
 #include "xf86_OSproc.h"
@@ -38,8 +38,8 @@ static const OptionInfoRec * BW2AvailableOptions(int chipid, int busid);
 static void	BW2Identify(int flags);
 static Bool	BW2Probe(DriverPtr drv, int flags);
 static Bool	BW2PreInit(ScrnInfoPtr pScrn, int flags);
-static Bool	BW2ScreenInit(int Index, ScreenPtr pScreen, int argc,
-			      char **argv);
+static Bool	BW2ScreenInit(int Index, ScreenPtr pScreen,
+			      const int argc, const char **argv);
 static Bool	BW2EnterVT(int scrnIndex, int flags);
 static void	BW2LeaveVT(int scrnIndex, int flags);
 static Bool	BW2CloseScreen(int scrnIndex, ScreenPtr pScreen);
@@ -386,7 +386,8 @@ BW2PreInit(ScrnInfoPtr pScrn, int flags)
 /* This gets called at the start of each server generation */
 
 static Bool
-BW2ScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
+BW2ScreenInit(int scrnIndex, ScreenPtr pScreen,
+	      const int argc, const char **argv)
 {
     ScrnInfoPtr pScrn;
     Bw2Ptr pBw2;
@@ -567,9 +568,9 @@ BW2ValidMode(int scrnIndex, DisplayModePtr mode, Bool verbose, int flags)
 /* Mandatory */
 static Bool
 BW2SaveScreen(ScreenPtr pScreen, int mode)
-    /* this function should blank the screen when unblank is FALSE and
-       unblank it when unblank is TRUE -- it doesn't actually seem to be
-       used for much though */
 {
-    return TRUE;
+    ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
+    Bw2Ptr pBw2 = GET_BW2_FROM_SCRN(pScrn);
+
+    return xf86SbusSaveScreen(pBw2->psdp, mode);
 }
