@@ -20,7 +20,7 @@
  * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
-/* $XFree86: xc/programs/Xserver/hw/xfree86/os-support/bus/Sbus.c,v 1.6 2004/12/07 21:53:46 tsi Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/os-support/bus/Sbus.c,v 1.15 2007/04/09 15:37:18 tsi Exp $ */
 
 #include <fcntl.h>
 #include <stdio.h>
@@ -41,12 +41,19 @@
 int promRootNode;
 
 static int promFd = -1;
-static int promCurrentNode;
 static int promOpenCount = 0;
 static int promP1275 = -1;
+
 #define MAX_PROP	128
 #define MAX_VAL		(4096-128-4)
+
+#if defined(__OpenBSD__) || defined(__NetBSD__)
+/* These two use their own OBP access scheme */
+static struct opiocdesc promOpio;
+static char promval[MAX_VAL + 1];
+#else
 static struct openpromio *promOpio;
+#endif
 
 sbusDevicePtr *xf86SbusInfo = NULL;
 
@@ -66,9 +73,65 @@ struct sbus_devtable sbusDeviceTable[] = {
     { SBUS_DEVICE_TCX, FBTYPE_TCXCOLOR, "tcx", "Sun TCX" },
     { SBUS_DEVICE_FFB, FBTYPE_CREATOR, "ffb", "Sun FFB" },
     { SBUS_DEVICE_FFB, FBTYPE_CREATOR, "afb", "Sun Elite3D" },
+    { SBUS_DEVICE_P9100, FBTYPE_P9100, "pnozz", "Weitek P9100" },
     { 0, 0, NULL, NULL }
 };
 
+#if defined(__OpenBSD__) || defined(__NetBSD__)
+int
+promGetSibling(int node)
+{
+    if (node == -1)
+	return 0;
+
+    promOpio.op_nodeid = node;
+    if (ioctl(promFd, OPIOCGETNEXT, &promOpio.op_nodeid) < 0)
+	return 0;
+
+    return promOpio.op_nodeid;
+}
+
+int
+promGetChild(int node)
+{
+    if (!node || (node == -1))
+	return 0;
+
+    promOpio.op_nodeid = node;
+    if (ioctl(promFd, OPIOCGETCHILD, &promOpio.op_nodeid) < 0)
+	return 0;
+
+    return promOpio.op_nodeid;
+}
+
+char *
+promGetProperty(const char *prop, int *lenp)
+{
+    /* op_nodeid has already been set */
+    promOpio.op_namelen = strlen(prop);
+    promOpio.op_name = (char *)prop;
+    promOpio.op_buflen = MAX_VAL;
+    promOpio.op_buf = promval;
+
+    if (ioctl(promFd, OPIOCGET, &promOpio) < 0)
+	return NULL;
+
+    if (lenp)
+	*lenp = promOpio.op_buflen;
+    return promOpio.op_buf;
+}
+
+int
+promGetBool(const char *prop)
+{
+    int length;
+
+    if (!promGetProperty(prop, &length) || (length < 0))
+	return 0;
+
+    return 1;
+}
+#else
 int
 promGetSibling(int node)
 {
@@ -80,8 +143,7 @@ promGetSibling(int node)
     if (ioctl(promFd, OPROMNEXT, promOpio) < 0)
 	return 0;
 
-    promCurrentNode = *(int *)promOpio->oprom_array;
-    return promCurrentNode;
+    return *(int *)promOpio->oprom_array;
 }
 
 int
@@ -95,8 +157,7 @@ promGetChild(int node)
     if (ioctl(promFd, OPROMCHILD, promOpio) < 0)
 	return 0;
 
-    promCurrentNode = *(int *)promOpio->oprom_array;
-    return promCurrentNode;
+    return *(int *)promOpio->oprom_array;
 }
 
 char *
@@ -105,7 +166,7 @@ promGetProperty(const char *prop, int *lenp)
     promOpio->oprom_size = MAX_VAL;
     strcpy(promOpio->oprom_array, prop);
     if (ioctl(promFd, OPROMGETPROP, promOpio) < 0)
-	return 0;
+	return NULL;
 
     if (lenp)
 	*lenp = promOpio->oprom_size;
@@ -128,6 +189,7 @@ promGetBool(const char *prop)
 	    return 1;
     }
 }
+#endif
 
 #define PROM_NODE_SIBLING 0x01
 #define PROM_NODE_PREF    0x02
@@ -164,11 +226,12 @@ promIsP1275(void)
 
     if (promP1275 != -1)
 	return;
-    promP1275 = 0;
+    promP1275 = FALSE;
     f = fopen("/proc/cpuinfo", "r");
     if (!f) return;
     while (fgets(buffer, 1024, f) != NULL)
-	if (!strncmp(buffer, "type", 4) && strstr(buffer, "sun4u")) {
+	if (!strncmp(buffer, "type", 4) &&
+	    (strstr(buffer, "sun4u") || strstr(buffer, "sun4v"))) {
 	    promP1275 = 1;
 	    break;
 	}
@@ -179,14 +242,20 @@ promIsP1275(void)
     if (promP1275 != -1)
 	return;
 
-    if ((uname(&buffer) == 0) && !strcmp(buffer.machine, "sun4u"))
+    if ((uname(&buffer) >= 0) &&
+        (!strcmp(buffer.machine, "sun4u") || !strcmp(buffer.machine, "sun4v")))
 	promP1275 = TRUE;
     else
 	promP1275 = FALSE;
-#elif defined(__FreeBSD__)
+#elif defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__)
+# if defined(__arch64__) || defined(__sparc_v9__) || defined(__sparc64__)
     promP1275 = TRUE;
+# else
+    /* Might need to check for a 32-bit userland running on a sun4u or sun4v */
+    promP1275 = FALSE;
+# endif
 #else
-#error Missing promIsP1275() function for this OS
+# error Missing promIsP1275() function for this OS
 #endif
 }
 
@@ -203,10 +272,12 @@ sparcPromClose(void)
 	promFd = -1;
     }
 
+#if !defined(__OpenBSD__) && !defined(__NetBSD__)
     if (promOpio) {
 	xfree(promOpio);
 	promOpio = NULL;
     }
+#endif
 
     promOpenCount = 0;
 }
@@ -219,16 +290,22 @@ sparcPromInit(void)
 	return 0;
     }
 
-    promFd = open("/dev/openprom", O_RDONLY, 0);
-    if (promFd == -1)
-	return -1;
-
-    promOpio = (struct openpromio *)xalloc(4096);
-    if (!promOpio) {
-	sparcPromClose();
-	return -1;
+    if (promFd < 0) {
+	promFd = open("/dev/openprom", O_RDONLY, 0);
+	if (promFd == -1)
+	    return -1;
     }
 
+#if !defined(__OpenBSD__) && !defined(__NetBSD__)
+    if (!promOpio) {
+	promOpio = (struct openpromio *)xalloc(4096);
+	if (!promOpio) {
+	    sparcPromClose();
+	    return -1;
+	}
+    }
+#endif
+
     promRootNode = promGetSibling(0);
     if (!promRootNode) {
 	sparcPromClose();
@@ -520,7 +597,7 @@ static int
 promWalkNode2Pathname(char *path, int parent, int node, int searchNode, int type)
 {
     int nextnode;
-    int len, ntype = type;
+    int len = 0, ntype = type;
     char *prop, *p;
 
     prop = promGetProperty("name", &len);
@@ -682,11 +759,58 @@ sparcPromPathname2Node(const char *pathName)
     return i;
 }
 
+Bool
+xf86LocateSbusMemoryArea(sbusDevicePtr psdp, char **devName,
+			 unsigned int *devOffset, unsigned int *fbSize,
+			 unsigned int *fbOffset, unsigned int *flags)
+{
+    /*
+     * Note that SBUS video drivers, contrary to other drivers, can be
+     * OS-specific.  That's because the offset(s) needed to mmap(2) SBUS video
+     * devices can be also.  One might think this common layer function could
+     * translate from a more generic offset scheme to the needed OS-specific
+     * one, but this would require adapter-specific knowledge in the common
+     * layer, and, for SBUS devices, there's enough of that already...
+     *
+     * Instead, this simply checks that all offsets are 32-bit.
+     */
+    unsigned int offset;
+
+    if (!psdp || !devOffset || devOffset[1])
+	return FALSE;
+
+    offset = devOffset[0];
+
+    if (fbOffset) {
+	if ((offset ^ (unsigned int)(-1L)) < *fbOffset)
+	    return FALSE;
+
+	offset += *fbOffset;
+    }
+
+    if (fbSize) {
+	if ((offset ^ (unsigned int)(-1L)) < *fbSize)
+	    return FALSE;
+    }
+
+    if (devName)
+	*devName = psdp->device;
+
+    if (flags)
+	*flags = 0;
+
+    return TRUE;
+}
+
 pointer
 xf86MapSbusMem(sbusDevicePtr psdp, unsigned long offset, unsigned long size)
 {
     pointer ret;
     unsigned long pagemask, off, len;
+    unsigned int devOffset[2], fbSize;
+#ifdef __OpenBSD__
+    static const unsigned int fbmode = WSDISPLAYIO_MODE_MAPPED;
+#endif
 
     if (!psdp || !size)
 	return NULL;
@@ -695,10 +819,23 @@ xf86MapSbusMem(sbusDevicePtr psdp, unsigned long offset, unsigned long size)
 	psdp->fd = open(psdp->device, O_RDWR);
 	if (psdp->fd == -1)
 	    return NULL;
+	psdp->mmapCount = 0;
+#ifdef __OpenBSD__
+	if (ioctl(psdp->fd, WSDISPLAYIO_SMODE, &fbmode) < 0)
+	    return NULL;
+#endif
     } else if (psdp->fd < 0) {
 	return NULL;
     }
 
+    devOffset[0] = offset;
+    devOffset[1] = 0;
+    fbSize = size;
+    if (!xf86LocateSbusMemoryArea(psdp, NULL, devOffset, &fbSize, NULL, NULL) ||
+	(devOffset[1] != 0))
+	return NULL;
+
+    offset = devOffset[0];
     pagemask = xf86getpagesize() - 1;
     off = offset & ~pagemask;
     len = ((offset + size + pagemask) & ~pagemask) - off;
@@ -712,6 +849,7 @@ xf86MapSbusMem(sbusDevicePtr psdp, unsigned long offset, unsigned long size)
 	    return NULL;
     }
 
+    psdp->mmapCount++;
     return (char *)ret + (offset - off);
 }
 
@@ -723,47 +861,15 @@ xf86UnmapSbusMem(sbusDevicePtr psdp, pointer addr, unsigned long size)
     unsigned long len = (((unsigned long)addr + size + mask) & ~mask) - base;
 
     munmap((pointer)base, len);
-}
 
-/* Tell OS that we are driving the HW cursor ourselves. */
-void
-xf86SbusHideOsHwCursor(sbusDevicePtr psdp)
-{
-    struct fbcursor fbcursor;
-    unsigned char zeros[8];
+    if (!--psdp->mmapCount) {
+#ifdef __OpenBSD__
+	static const unsigned int fbmode = WSDISPLAYIO_MODE_EMUL;
 
-    memset(&fbcursor, 0, sizeof(fbcursor));
-    memset(&zeros, 0, sizeof(zeros));
-    fbcursor.cmap.count = 2;
-    fbcursor.cmap.red = zeros;
-    fbcursor.cmap.green = zeros;
-    fbcursor.cmap.blue = zeros;
-    fbcursor.image = (char *)zeros;
-    fbcursor.mask = (char *)zeros;
-    fbcursor.size.x = 32;
-    fbcursor.size.y = 1;
-    fbcursor.set = FB_CUR_SETALL;
-    ioctl(psdp->fd, FBIOSCURSOR, &fbcursor);
-}
-
-/* Set HW cursor colormap. */
-void
-xf86SbusSetOsHwCursorCmap(sbusDevicePtr psdp, int bg, int fg)
-{
-    struct fbcursor fbcursor;
-    unsigned char red[2], green[2], blue[2];
-
-    memset(&fbcursor, 0, sizeof(fbcursor));
-    red[0] = bg >> 16;
-    green[0] = bg >> 8;
-    blue[0] = bg;
-    red[1] = fg >> 16;
-    green[1] = fg >> 8;
-    blue[1] = fg;
-    fbcursor.cmap.count = 2;
-    fbcursor.cmap.red = red;
-    fbcursor.cmap.green = green;
-    fbcursor.cmap.blue = blue;
-    fbcursor.set = FB_CUR_SETCMAP;
-    ioctl(psdp->fd, FBIOSCURSOR, &fbcursor);
+	/* Ignore errors */
+	ioctl(psdp->fd, WSDISPLAYIO_SMODE, &fbmode);
+#endif
+	close(psdp->fd);
+	psdp->fd = -1;
+    }
 }
