@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/vbe/vbe.c,v 1.19 2006/03/16 16:50:35 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/vbe/vbe.c,v 1.20 2008/07/16 17:01:43 tsi Exp $ */
 
 /*
  *                   XFree86 vbe module
@@ -989,6 +989,9 @@ VBESetGetPaletteData(vbeInfoPtr pVbe, Bool set, int first, int num,
 	AX    := VBE Return Status
      */
 
+    if (set && !data)
+	return NULL;
+
     pVbe->pInt10->num = 0x10;
     pVbe->pInt10->ax = 0x4f09;
     if (!secondary)
