@@ -21,7 +21,7 @@
  * used in advertising or publicity pertaining to distribution of the software
  * without specific, written prior permission.
  */
-/* $XFree86: xc/programs/xmh/xmh.h,v 1.4 2006/01/09 15:01:40 dawes Exp $ */
+/* $XFree86: xc/programs/xmh/xmh.h,v 1.5 2006/12/10 15:58:35 tsi Exp $ */
 
 #ifndef _xmh_h
 #define _xmh_h
@@ -61,6 +61,8 @@
 #define xMargin 2
 #define yMargin 2
 
+#undef DEBUG
+
 #define DEBUG(msg) \
 	if (app_resources.debug) \
 	    {(void)fprintf(stderr, msg); (void)fflush(stderr);}
